# nix-darwin

Nix modules for darwin, `/etc/nixos/configuration.nix` for macOS.
This will creates and manages a system profile in `/run/current-system`, just like nixos.

```
$ darwin-rebuild switch
building the system configuration...
these derivations will be built:
  /nix/store/vfad6xgjzr56jcs051cg6vzch4dby92y-etc-zprofile.drv
  /nix/store/cbmkscxsz0k02ynaph5xaxm1aql0p3vq-etc.drv
  /nix/store/r5fpn177jhc16f8iyzk12gcw4pivzpbw-nixdarwin-system-16.09.drv
building path(s) ‘/nix/store/wlq89shja597ip7mrmjv7yzk2lwyh8n0-etc-zprofile’
building path(s) ‘/nix/store/m8kcm1pa5j570h3indp71a439wsh9lzq-etc’
building path(s) ‘/nix/store/l735ffcdvcvy60i8pqf6v00vx7lnm6mz-nixdarwin-system-16.09’
writing defaults...
setting up /etc...
warning: /etc/zprofile is a file, skipping...
warning: /etc/zshrc is a file, skipping...
setting up launchd services...
$ 
```

## Install

This will link the system profile to `/run/current-system`, you have to create `/run` or symlink it to `private/var/run`.
If you use a symlink you'll probably also want to add `services.activate-system.enable = true;` to your configuration.

> NOTE: the system activation scripts don't overrwrite existing etc files, etc files like `/etc/bashrc` won't be used by default.
Either modify the existing file to source/import the one from `/etc/static` or remove it.

```bash
git clone git@github.com:LnL7/nix-darwin.git
nix-build -I darwin=$PWD/nix-darwin -I darwin-config=$PWD/config.nix '<darwin>' -A system
source result/etc/bashrc

result/sw/bin/darwin-rebuild build
result/sw/bin/darwin-rebuild switch
```

If you already configured your `NIX_PATH` you can just (re)bootstrap nix-darwin

```bash
$(nix-build '<darwin>' -A system --no-out-link)/sw/bin/darwin-rebuild switch
```

## Example configuration

Checkout [modules/examples](https://github.com/LnL7/nix-darwin/tree/master/modules/examples) for some example configurations.

```nix
{ config, lib, pkgs, ... }:
{

  # List packages installed in system profile. To search by name, run:
  # $ nix-env -qaP | grep wget
  environment.systemPackages =
    [ pkgs.nix-repl
    ];

  programs.bash.enable = true;
  programs.bash.interactiveShellInit = ''
    # Edit the NIX_PATH entries below or put the nix-darwin repository in
    # ~/.nix-defexpr/darwin and your configuration in ~/.nixpkgs/darwin-config.nix

    export NIX_PATH=darwin=$HOME/.nix-defexpr/darwin:darwin-config=$HOME/.nixpkgs/darwin-config.nix:$NIX_PATH
  '';

  services.activate-system.enable = true;

}
```

## Modules

### system.defaults

A set of [modules](https://github.com/LnL7/nix-darwin/tree/master/modules/system/defaults) to manage macOS settings.

```nix
{
  system.defaults.dock.autohide = true;
  system.defaults.dock.orientation = "left";
  system.defaults.dock.showhidden = true;
}
```

> NOTE: you have to restart the dock in order for these changes to apply. `killall Dock`

### environment.etc

Set of files to be linked in `/etc`, this won't overwrite any existing files.
Either modify the existing file to source/import the one from `/etc/static` or remove it.

```nix
{
  environment.etc."foorc".text = ''
    export FOO=1

    if test -f /etc/foorc.local; then
      . /etc/foorc.local
    fi
  '';
}
```

### launchd.daemons

Definition of launchd agents/daemons. The [serviceConfig](https://github.com/LnL7/nix-darwin/blob/master/modules/launchd/launchd.nix) options are used to generate the launchd plist file.

```nix
{
  launchd.daemons.foo = {
    serviceConfig.ProgramArguments = [ "/usr/bin/touch" "/tmp/foo.lock" ];
    serviceConfig.RunAtLoad = true;
  };
}
```

### services

A set of modules for predefined services, these generate the appropriate launchd daemons for you.

```nix
{
  services.nix-daemon.enable = true;
  services.nix-daemon.tempDir = "/nix/tmp";
}
```

### programs

A set of modules to manage configuration of certain programs.

```nix
{
  environment.shellAliases.tm = "${pkgs}/bin/tmux";

  programs.bash.enable = true;

  programs.tmux.enable = true;
  programs.tmux.enableSensible = true;
  programs.tmux.loginShell = "${config.programs.bash.shell} -l";
}
```

### nixpkgs.config

This attribute set is used as nixpkgs configuration when using nix-darwin.

```nix
{
  environment.systemPackages =
    [ # Use vim_configurable from packageOverrides
      lnl.vim
    ];

  nixpkgs.config.allowUnfree = true;

  nixpkgs.config.packageOverrides = pkgs: {
    lnl.vim = pkgs.vim_configurable.customize {
      name = "vim";
      vimrcConfig.customRC = ''
        set nocompatible
        filetype plugin indent on
        syntax on
      '';
    };
  };
}
```
